--[[
Copyright 2010, 2011 João Cardoso
Cornucopia is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Cornucopia.

Cornucopia is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cornucopia is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cornucopia. If not, see <http://www.gnu.org/licenses/>.
--]]

local Actions = Cornucopia:CreateGroup('Actions', {
	templates = 'SecureHandlerStateTemplate',
	sets = 'Cornucopia_Actions_Sets',
	name = 'Action Buttons',
	defaults = {
		{
			point = 'BOTTOM',
			x = -254, y = 2,
			states = {
				['bonusbar:5'] = true,
				['stances'] = true,
			},
			columns = 12, rows = 1,
			start = '1:1', spacing = 3,
			ids = {}
		},
	},
	barDefaults = {
		point = 'CENTER',
		ids = {}, states = {},
		start = '1:1', rows = 1,
		spacing = 3,
	}
})

local L, IDs = Cornucopia_Locals, {}
local Stances, StancesIcon, StancesTitle, Stealth
local class = select(2, UnitClass('player'))

if class == 'DRUID' then
	Stances, StancesIcon, StancesTitle, Stealth = {'stance:1', 'stance:3'}, 768, TUTORIAL_TITLE61_DRUID, true
elseif class == 'WARRIOR' then
	Stances, StancesIcon, StancesTitle = {'stance:2', 'stance:3'}, 2457, TUTORIAL_TITLE61_WARRIOR
elseif class == 'ROGUE' then
	Stances, StancesIcon, Stealth = {'stance:3'}, 1784, true
elseif class == 'PRIEST' then
	Stances, StancesIcon = {'stance:1'}, 15473
elseif class == 'WARLOCK' then
	Stances, StancesIcon = {'stance:2'}, 59672
else
	Stances = {}
end

for i = 1, 120 do
	IDs[i] = i
end


--[[ Startup ]]--

function Actions:OnInitialize()
	self.frame:SetAttribute('_onstate-pages', [[control:ChildUpdate('pages', newstate)]])
	self.OnLock = self.OnUnlock
	self:Update()
	
	for _,id in pairs(self.sets.ids) do
		for index,i in pairs(IDs) do
			if id == i then
				tremove(IDs, index)
				break
			end
		end
	end
end

function Actions:OnRemove()
	local sets = self.sets
	for _,id in pairs(sets.ids) do
		tinsert(IDs, id)
	end
	sort(IDs)
	
	for i = 1, sets.rows * sets.columns do
		self:RemoveButton(i)
	end
end

function Actions:Update()
	local sets = self.sets
	local bottom, left = strsplit(':', sets.start)
	local columns, rows = sets.columns, sets.rows
	local numButtons = columns * rows
	local spacing = sets.spacing
	local size = 39 + spacing
	local ids = sets.ids
	
	self.frame:SetAttribute('state-pages', '0') -- Force Update
	self.frame:SetWidth(size * columns - spacing)
	self.frame:SetHeight(size * rows - spacing)
	
	self:UpdateStates()
	
	local numPages = self.pages
	if #ids == 0 then
		for i = 1, numPages * numButtons do
			ids[i] = tremove(IDs, 1)
		end
	end
	
	for i = 1, numButtons do
		local x = mod(i - 1, columns)
		local y = ceil(i / columns) - 1
		
		if bottom == '1' then x = columns - 1 - x end
		if left == '1' then y = rows - 1 - y end
		
		local button = self:GetButton(i) or self:NewButton(i)
		button:SetPoint('TOPRIGHT', self.frame, -x * size - 2, -y * size - 2)
		button:SetAttribute('statehidden', nil)
		button:SetAttribute('id0', 120 + i)
		button:Show()
		
		for page = 1, numPages do
			button:SetAttribute('id'..page, ids[numButtons * (page - 1) + i])
		end
	end
	
	RegisterStateDriver(self.frame, 'pages', self.states)
end


--[[ Buttons ]]--

function Actions:NewButton(i)
	local button = CreateFrame('CheckButton', 'CornucopiaAction'..i..'Bar'..self.index, self.frame, 'ActionBarButtonTemplate')
	button:SetID(0)
	button:SetAttribute('_childupdate-pages', [[
		self:SetAttribute('action', self:GetAttribute('id'..message))
	]])
	
	return button
end

function Actions:RemoveButton(i)
	local button = self:GetButton(i)
	button:SetAttribute('statehidden', true)
	button:Hide()
				
	local key = GetBindingKey('CLICK '..button:GetName()..':LeftButton')
	if key then
		SetBinding(key, nil)
		SaveBindings(GetCurrentBindingSet())
	end
end

function Actions:GetButton(i)
	return _G['CornucopiaAction'..i..'Bar'..self.index]
end


--[[ States ]]--

function Actions:UpdateStates()
	self.pages = 1
	
	local states = self:AddState('mod:ctrl') .. self:AddState('mod:shift') .. self:AddState('mod:alt')
	local stances, stealth = '', ''
	
	-- Stealth
	if Stealth then
		stealth = self:AddState('stealth', states, 'stances')
	end
	
	-- Situations
	states = self:AddState('group:raid', states) .. states
	states = self:AddState('combat', states) .. states
	
	--Stances
	for _, stance in pairs(Stances) do
		stances = self:AddState(stance, states, 'stances') .. stances
	end
	stances =  self:AddState('help', states) .. stances

	self.states = self:AddState('bonusbar:5', nil, nil, '0') .. stealth .. stances .. states .. 1
end

function Actions:AddState(state, original, arg, page)
	if self.sets.states[(arg or state)] then
		local add = ''
		for orig in gmatch(original or '', '%[([^;]+)%]') do
			add = add .. self:GetState(state .. ', ' .. orig)
		end
		return add .. self:GetState(state, page)
	end
	return ''
end

function Actions:GetState(state, page)
	if not page then
		self.pages = self.pages + 1
	end
	return '['..state..']'..(page or self.pages)..';'
end


--[[ Config ]]--

function Actions:OnAddAttempt()
	self.barDefaults.columns = min(3, #IDs)
	self.cannotAdd = #IDs == 0
end

function Actions:OnOptionsShown()
	LibStub('Sushi-2.0'):Embed(self)
	local sets = self.sets
	local ids = sets.ids
	
	local Layout, States = self:CreatePanel('Layout'), self:CreatePanel('Events')
	local Help = self:CreateWidget('HelpButton', States)
	Help:SetDescription('These options allow you to make the actions in your bar to change in some situations.')
	Help:SetPoint('TOPRIGHT', 8, 3)
	
	local UsedIds = CreateFrame('StatusBar', nil, States)
	UsedIds:SetStatusBarTexture('Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar')
	UsedIds:SetStatusBarColor(.25, .25, .75)
	UsedIds:SetPoint('BOTTOMLEFT', 8.5, 3)
	UsedIds:SetMinMaxValues(0, 120)
	UsedIds:SetSize(188, 12)
	
	local BarIds = CreateFrame('StatusBar', nil, UsedIds)
	BarIds:SetStatusBarTexture('Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar')
	BarIds:SetStatusBarColor(.25, .75, .25)
	BarIds:SetMinMaxValues(0, 120)
	BarIds:SetAllPoints()
	
	local IdsText = BarIds:CreateFontString(nil, nil, 'GameFontHighlightSmall')
	IdsText:SetPoint('CENTER')
	
	local LeftBorder = BarIds:CreateTexture(nil, 'OVERLAY')
	LeftBorder:SetTexture('Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder')
	LeftBorder:SetTexCoord(1, .3983, 1, 0)
	LeftBorder:SetPoint('LEFT', -3, 0)
	LeftBorder:SetSize(97.58, 27)
	
	local RightBorder = BarIds:CreateTexture(nil, 'OVERLAY')
	RightBorder:SetTexture('Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder')
	RightBorder:SetPoint('LEFT', LeftBorder, 'RIGHT', -1, 0)
	RightBorder:SetTexCoord(.3983, 1, 1, 0)
	RightBorder:SetSize(97.58, 27)
	
	-- Layout
	do
		local function Create(kind, label, arg, method)
			local arg = arg or strlower(label)
			local widget = Layout:CreateChild(kind)
			widget:SetLabel(L[label] or label)
			widget:SetValue(sets[arg])
			
			widget:RegisterCallback('OnValueChanged', function(v)
				if method then
					method(v)
				end
			
				self.sets[arg] =  v
				self:Update()
			end)
			return widget
		end
		
		local function CreateSlider(label, min, max, method)
			local slider = Create('Slider', label, nil, method)
			slider:SetRange(min, max)
			return slider
		end
		
		local function UpdateSize(new)
			local old = sets.rows * sets.columns
			local numPages = self.pages
			
			for page = 1, numPages do
				for i = old + 1, new do
					tinsert(ids, new * (page - 1) + i, tremove(IDs, 1))
				end
			end
			
			for page = numPages, 1, -1 do
				for i = old, new + 1, -1 do
					tinsert(IDs, 1, tremove(ids, old * (page - 1) + i))
				end
			end
			
			for i = new + 1, old do
				self:RemoveButton(i)
			end
		end
	
		Layout:SetChildren(function()
			local Start = Create('Dropdown', 'Start From', 'start')
			Start:AddLine(':', L.TopRight)
			Start:AddLine('1:', L.TopLeft)
			Start:AddLine(':1', L.BottomRight)
			Start:AddLine('1:1', L.BottomLeft)
			
			local columns, rows = sets.columns, sets.rows
			local freeRatio = (#IDs + #ids) / self.pages
			
			CreateSlider('Columns', 1, min(20, floor(freeRatio / rows)), function(v) UpdateSize(v * rows) end)
			CreateSlider('Rows', 1, min(20, floor(freeRatio / columns)), function(v) UpdateSize(v * columns) end)
			CreateSlider('Spacing', 0, 10).insets = {bottom = 10}
		end)
	end
	
	-- States
	do
		local function AddIds(old)
			local numButtons = sets.rows * sets.columns
			local newPages = self.pages
			local new = self.states
			
			for page = 2, newPages do
				local state = strmatch(new, '%[([^;]+)%]'..page..';')
				if state and not strmatch(old, state) then
					for i = 1, numButtons do
						tinsert(ids, numButtons * (page - 1) + 1, tremove(IDs, 1))
					end
				end
			end
		end
		
		local function RemoveIds(old, oldPages)
			local numButtons = sets.rows * sets.columns
			local new = self.states

			for page = oldPages, 2, -1 do
				local state = strmatch(old, '%[([^;]+)%]'..page)
				if state and not strmatch(new, state) then
					for i = 1, numButtons do
						tinsert(IDs, 1, tremove(ids, numButtons * (page - 1) + 1))
					end
				end
			end
			sort(IDs)
		end
		
		local function CanEnable(arg)
			sets.states[arg] = true
			self:UpdateStates()
			
			local can = (sets.rows * sets.columns * self.pages) < (#IDs + #ids)
			
			sets.states[arg] = nil
			self:UpdateStates()
			
			return can
		end
		
		local function Create(text, icon, arg)
			local value = sets.states[arg]
			local enabled = value or CanEnable(arg)
			
			local check = States:CreateChild('IconCheck')
			check:SetDescription(L[text..'Desc'])
			check:SetText(L[text] or text)
			check:SetDisabled(not enabled)
			check:SetChecked(value)
			check:SetIcon(icon)
			
			check:RegisterCallback('OnValueChanged', function(enable)
				local states, pages = self.states, self.pages
				sets.states[arg] = enable
				
				self:UpdateStates()
				if enable then
					AddIds(states, pages)
				else
					RemoveIds(states, pages)
				end
				self:Update()
			end)
			return check
		end
		
		States:SetChildren(function()
			if StancesIcon then
				Create(StancesTitle or GetSpellInfo(StancesIcon), GetSpellTexture(StancesIcon), 'stances')
			end
			
			Create('Vehicle & Possess', 'Interface\\Icons\\Inv_Misc_Wrench_02', 'bonusbar:5')
			Create('Friendly Target', 'Interface\\Icons\\Spell_Holy_DivineProvidence', 'help')
			Create(COMBAT, 'Interface\\Icons\\Ability_Warrior_Challange', 'combat')
			Create(GROUP, 'Interface\\Icons\\INV_Misc_GroupNeedMore', 'group:raid')
			
			Create('Ctrl', 'Interface\\Addons\\Cornucopia\\Art\\Keyboard', 'mod:ctrl')
			Create('Shift', 'Interface\\Addons\\Cornucopia\\Art\\Keyboard', 'mod:shift')
			Create('Alt', 'Interface\\Addons\\Cornucopia\\Art\\Keyboard', 'mod:alt').insets = {bottom = 20}
			
			local used = 120 - #IDs
			IdsText:SetText(used .. ' / 120')
			UsedIds:SetValue(used)
			BarIds:SetValue(#ids)
		end)
	end
	
	self.options = {Layout, States}
end

function Actions:OnToolsShown()
	self.tools = CornucopiaActionsBinder or CornucopiaActions_StartBinder()
	self.OnDoubleClick = self.tools.OnClick
	self.UpdateBinder = self.tools.Update
	self.OnToolsShown = nil
end

function Actions:OnUnlock()
	if self.UpdateBinder then
		self:UpdateBinder()
	end
	
	for i = 1, self.frame:GetNumChildren() do
		local button = select(i, self.frame:GetChildren())
		button:SetAttribute('showgrid', button:GetAttribute('showgrid') + (self.unlocked and 1 or -1))
		
		if self.unlocked then
			ActionButton_ShowGrid(button)
		else
			ActionButton_HideGrid(button)
		end
	end
end

function Actions:CreatePanel(name)
	local frame = self:CreateWidget('Container')
	frame:SetLayout('Flow')
	frame:SetHeight(nil)
	frame.text = name
	return frame
end