--[[
Copyright 2010, 2011 João Cardoso
Cornucopia is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Cornucopia.

Cornucopia is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cornucopia is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cornucopia. If not, see <http://www.gnu.org/licenses/>.
--]]

local L = Cornucopia_Locals
local Debuffs = Cornucopia:CreateBar('Debuffs', {
	type = 'Movable Bar',
	sets = 'Cornucopia_Debuffs_Sets',
	actualDisplay = 'DEBUFF_ACTUAL_DISPLAY',
	maxDisplay = 'DEBUFF_MAX_DISPLAY',
	buttonsName = 'DebuffButton',
	filter = 'HARMFUL',
	name = 'Debuffs',
	buttons = {},
	defaults = {
		point = 'TopRight', relPoint = 'BottomRight',
		anchor = 'Buffs', x = 0, y = 0,
		columns = 8, rows = 2,
		start = ':',
		spacing = 5,
	}
})


--[[ Startup ]]--

function Debuffs:OnInitialize()
	DebuffButton_UpdateAnchors = function() end
	
	self.UpdateFrame = BuffFrame_Update
	self:OnSettingsChanged()
	
	function BuffFrame_Update()
		self.UpdateFrame()
		self:UpdateSize()
		
		for i = 1, DEBUFF_ACTUAL_DISPLAY do
			self.buttons[i] = _G['DebuffButton'..i]
		end
		self:PlaceButtons()
	end
end

function Debuffs:OnSettingsChanged()
	local maxDisplay, buttonsName = self.maxDisplay, self.buttonsName
	local new = self.sets.columns * self.sets.rows
	for i = new, _G[maxDisplay] do
		local button = _G[buttonsName..i]
		if button then
			button:Hide()
		end
	end
	
	_G[maxDisplay] = new
	BuffFrame_Update()
end


--[[ API ]]--

function Debuffs:UpdateSize()
	local columns, rows, width, height, spacing = self:GetSizes()
	local actualDisplay = _G[self.actualDisplay]
	
	self.frame:SetHeight(ceil(actualDisplay / columns) * height - spacing + 4)
	self.frame:SetWidth(min(actualDisplay, columns) * width - spacing + 4)
end

function Debuffs:PlaceButtons(index)
	local buttons = self.buttons
	sort(buttons, function(A, B)
		if not B or A == B then
			return nil
		elseif not A or not A.timeLeft then
			return true
		end
		
		return B.timeLeft and A.timeLeft > B.timeLeft
	end)
	
	for i, button in pairs(buttons) do
		self:PlaceButton(button, i + (index or 0))
	end
	wipe(buttons)
end

function Debuffs:PlaceButton(button, index)
	local columns, rows, width, height, spacing, bottom, left = self:GetSizes()
	local y = ceil(index / columns) - 1
	local x = mod(index - 1, columns)
	
	if bottom == '1' then x = columns - 1 - x end
	if left == '1' then y = rows - 1 - y end
	
	button:ClearAllPoints()
	button:SetParent(self.frame)
	button:SetPoint('TOPRIGHT', -x * width - 2, -y * height - 2)
end

function Debuffs:GetSizes()
	local spacing = self.sets.spacing
	local width = 30 + spacing
	return self.sets.columns, self.sets.rows, width, width + BUFF_ROW_SPACING - 5, spacing, strsplit(':', self.sets.start)
end


--[[ Unlock ]]--

function Debuffs:OnLock()
	BuffFrame_Update()
end

function Debuffs:OnUnlock()
	if not self.UnitAura then
		self.UnitAura = UnitAura
		
		function UnitAura(unit, index, filter)
			if self.unlocked and unit == PlayerFrame.unit and filter == self.filter then
				local total = _G[self.maxDisplay]
				return '', '', 'Interface\\Icons\\Temp', index, nil, 10 + 3590 / (total - 1) * (total - index), GetTime() + 10^10 - index
			end
			return self.UnitAura(unit, index, filter)
		end
		
		hooksecurefunc('AuraButton_UpdateDuration', function(button)
			if self.unlocked and button.filter == self.filter then
				local time = select(6, UnitAura(button.unit, button:GetID(), button.filter))
				button.duration:SetFormattedText(SecondsToTimeAbbrev(time))
			end
		end)
	end
	
	BuffFrame_Update()
end


--[[ Options ]]--

function Debuffs:OnOptionsShown()
	local Options = LibStub('Sushi-2.0'):CreateWidget('Container')
	Options.text = 'Layout'
	Options:SetLayout('Flow')
	Options:SetHeight(nil)
		
	local function Create(kind, label, arg)
		local arg = arg or strlower(label)
		local widget = Options:CreateChild(kind)
		widget:SetLabel(L[label] or label)
		widget:SetValue(self.sets[arg])
		widget:RegisterCallback('OnValueChanged', function(v)
			self.sets[arg] =  v
			self:OnSettingsChanged()
		end)
		return widget
	end
	
	Options:SetChildren(function()
		local Start = Create('Dropdown', 'Grow From', 'start')
		Start:AddLine(':', L.TopRight)
		Start:AddLine('1:', L.TopLeft)
		Start:AddLine(':1', L.BottomRight)
		Start:AddLine('1:1', L.BottomLeft)
		
		Create('Slider', 'Columns'):SetRange(1, 20)
		Create('Slider', 'Rows'):SetRange(1, 20)
		
		local Spacing = Create('Slider', 'Spacing')
		Spacing.insets = {bottom = 10}
		Spacing:SetRange(0, 10)
	end)
	
	self.OnOptionsShown = nil
	self.options = {Options}
end