--[[
Copyright 2010, 2011 João Cardoso
Cornucopia is distributed under the terms of the GNU General Public License (or the Lesser GPL).
This file is part of Cornucopia.

Cornucopia is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cornucopia is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cornucopia. If not, see <http://www.gnu.org/licenses/>.
--]]


local GetPosition, GetPoint, IsDependant, IteratePoints, CodePoint, DecodePoint = Cornucopia:ProvideUtility()
local Metrics = LibStub('Sushi-2.0'):CreateWidget('Container')
local L = Cornucopia_Locals
local bar, data, sets


--[[ Startup ]]--

function Metrics:Startup()
	self:Hide()
	self:SetWidth(250)
	self:SetHeight(nil)
	self:SetLayout('Flow')
	self:SetChildren('Update')
	
	self.name = 'Metrics'
	self.Startup = nil
end

function Metrics:Update()
	local parent = self:GetParent()
	if parent then
		bar = parent.bar
	end

	if bar then
		data = bar.data
		sets = bar.sets
	
		-- Anchor
		self:AnchorDropdown()
		self:PointsDropdown('With', 'point')
		self:PointsDropdown('To', 'relPoint')
		self:OffsetEditBox('X', 'x')
		self:OffsetEditBox('Y', 'y')
		
		-- Size
		self:ScaleSlider()
		if data.width or data.height then
			self:SizeSlider('Height', 'height', 'SetHeight', data.minHeight, GetScreenHeight())
			self:SizeSlider('Width', 'width', 'SetWidth', data.minWidth, GetScreenWidth())
		end
		
		-- Opacity
		self:OpacitySlider()
	end
end


--[[ Common ]]--

function Metrics:HeaderChild(kind, name, width)
	local header = self:CreateChild('Header')
	header:SetFont('GameFontHighlight')
	header:SetUnderlined(true)
	header:SetText(L[name])

	local child = self:CreateChild(kind)
	child.insets = {top = -15}
	child:SetWidth(width)
	return child
end

function Metrics:SmallChild(kind, width, name)
	local child = self:CreateChild(kind)
	child:SetLabel(L[name])
	child:SetWidth(width)
	child:SetSmall(true)
	return child
end


--[[ Anchor ]]--

function Metrics:AnchorDropdown()
	local t = {}
	local drop = self:HeaderChild('Dropdown', 'Anchor', 260)
	drop:SetValue(bar.sets.anchor or UIParent)
	drop:AddLine(UIParent, L['Screen'])
	
	for id, target in Cornucopia:IterateBars() do
		if not IsDependant(target, bar) then
			tinsert(t, target.data.name)
		end
	end
	sort(t)
	
	for _, name in pairs(t) do
		for id, target in Cornucopia:IterateBars() do
			if target.data.name == name then
				drop:AddLine(id, name)
				break
			end
		end
	end
	
	drop:RegisterCallback('OnValueChanged', function(v)
		bar.sets.anchor = v ~= UIParent and v or nil
		bar:CalculatePosition()
	end)
end

function Metrics:PointsDropdown(name, arg)
	local drop = self:SmallChild('Dropdown', 140, name, arg)
	drop.insets = {bottom = 12, right = -15}
	
	drop:SetValue(bar.sets[arg] or bar.sets['point'] or '')
	drop:RegisterCallback('OnValueChanged', function(v)
		bar.sets[arg] = v
		bar:CalculatePosition()
	end)
	
	for x, y in IteratePoints() do
		local point = CodePoint(y..x)
		drop:AddLine(point, L[point])
	end
end

function Metrics:OffsetEditBox(name, arg)
	local box = self:SmallChild('EditBox', 97, name, arg)
	local value = (bar.sets[arg] or 0) * 100 + 0.5
	
	box:SetCursorPosition(0)
	box:SetValue(floor(value) / 100)
	box.insets = {top = -15, bottom = 12, right = -15}
	box:RegisterCallback('OnValueChanged', function(v)
		if tonumber(v) then
			bar.sets[arg] = v
			bar:UpdatePosition()
		end
	end)
end


--[[ Size & Opacity ]]--

function Metrics:ScaleSlider()
	local maximun = min(GetScreenHeight() / bar:GetHeight(), GetScreenWidth() / bar:GetWidth(), data.maxScale or 3)
	local slider = self:PercentSlider('Size', sets.scale, data.minScale or 0.2, maximun, function(scale)
		sets.scale = scale
		bar:CalculatePosition()
		bar:SetScale(scale)
	end)
end

function Metrics:OpacitySlider()
	local slider = self:PercentSlider('Opacity', sets.alpha, 0, 1, function(alpha)
		bar:SetAlpha(alpha)
		sets.alpha = alpha
	end)
end

function Metrics:PercentSlider(name, value, min, max, method)
	local slider = self:HeaderChild('Slider', name, 230)
	slider:SetRange(min * 100, max * 100)
	slider:SetValue((value or 1) * 100)
	slider:SetRangeText('', '')
	slider:SetPattern('%s%')
	slider.insets.bottom = 9
	
	slider:RegisterCallback('OnValueChanged', function(v)
		method(v / 100)
	end)
end

function Metrics:SizeSlider(name, arg, method, min, max)
	local slider = self:SmallChild('Slider', 110, name, arg)
	local scale = bar:GetScale()

	slider.insets = {right = -20, bottom = 11, top = -9}
	slider:SetRange((min or 5) / scale, max / scale)
	slider:SetValueText(not data[arg] and '')
	slider:SetDisabled(not data[arg])
	slider:SetValue(sets[arg] or 0)
	slider:SetRangeText('', '')
	
	slider:RegisterCallback('OnValueChanged', function(v)
		sets[arg] = v
		bar[method](bar, v)
	end)
end


--[[ The End ]]--

CornucopiaInspector.Metrics = Metrics
Metrics:Startup()